"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/tolerance.py $
$Id: tolerance.py 25070 2004-09-10 18:02:07Z dbinger $
"""

from dulcinea.base import DulcineaBase
from dulcinea.typeutils import typecheck

class Tolerance (DulcineaBase):
    """
    A tolerance value can be associated with another value (e.g. a
    Parameter) in order to specify the relative tolerance (in percent) of
    that value.

    Instance attributes:
      hi : float
      lo : float
    """

    def __init__(self, hi, lo=None):
        assert hi >= 0
        typecheck(hi, float)
        if lo is None:
            lo = hi
        else:
            assert lo >= 0
            typecheck(lo, float)
        self.hi = hi
        if abs(hi - lo) < 1e-9:
            lo = hi # avoid having to compare with fudge elsewhere
        self.lo = lo

    def __str__(self):
        return self.format()

    def format(self, html=0):
        if html:
            plusminus = '&plusmn;' # a one character version of +/-
        else:
            plusminus = '+/-'
        if self.hi == self.lo:
            return '%s%g%%' % (plusminus, self.hi)
        else:
            return '+%g%%/-%g%%' % (self.hi, self.lo)

    def __hash__(self):
        return hash((self.lo, self.hi))

    def is_symmetric(self):
        """Return true if the high and low magnitudes are the same."""
        return self.hi == self.lo

    def get_hi(self):
        return self.hi

    def get_lo(self):
        return self.lo
